#include <bits/stdc++.h>

const int32_t MAX_N = 3e5;

struct SegmentTree {
	int32_t data[8 * MAX_N + 5];

	void update(int32_t node, int32_t low, int32_t high, int32_t qInd) {
		if(low > qInd || high < qInd) {
			return;
		}
		else if(low == qInd && high == qInd) {
			data[node] ^= 1;
			return;
		}

		int32_t mid = (low + high) / 2;
		update(2 * node, low, mid, qInd);
		update(2 * node + 1, mid + 1, high, qInd);

		data[node] = data[2 * node] + data[2 * node + 1];
	}

	int32_t query(int32_t node, int32_t low, int32_t high, int32_t qLow, int32_t qHigh) {
		if(low > qHigh || high < qLow) {
			return 0;
		}
		else if(low >= qLow && high <= qHigh) {
			return data[node];
		}

		int32_t mid = (low + high) / 2;
		return query(2 * node, low, mid, qLow, qHigh) + query(2 * node + 1, mid + 1, high, qLow, qHigh);
	}
};

struct Vertex {
	bool infected;
	int32_t p, in, out;	
	std::vector< int32_t > children;
};

Vertex g[MAX_N + 5];
SegmentTree segTree;

void dfs(int32_t u, int32_t &t) {
	g[u].in = t++;
	
	for(auto &v : g[u].children) {
		dfs(v, t);
	}

	g[u].out = t++;
}

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr);

	int32_t n;
	std::cin >> n;

	for(int32_t i = 2; i <= n; i++) {
		std::cin >> g[i].p;
		g[g[i].p].children.push_back(i);
	}

	int32_t t = 1;
	dfs(1, t);

	int32_t q;
	std::cin >> q;

	for(int32_t i = 0; i < q; i++) {
		int32_t type, x;
		std::cin >> type >> x;

		if(type == 1) {
			if(!g[x].infected) {
				g[x].infected = true;
				segTree.update(1, 1, 2 * n, g[x].in);
			}
		}
		else if(type == 2) {
			int32_t last = -1;
			while(1) {
				if(g[x].infected) {
					for(auto &y : g[x].children) {
						if(y != last && !g[y].infected) {
							g[y].infected = true;
							segTree.update(1, 1, 2 * n, g[y].in);
						}
					}
					g[x].infected = false;
					segTree.update(1, 1, 2 * n, g[x].in);
				}

				if(x == 1) {
					break;
				}

				last = x;
				x = g[x].p;
			}
		}
		else if(type == 3) {
			if(g[x].infected) {
				g[x].infected = false;
				segTree.update(1, 1, 2 * n, g[x].in);
			}

			for(auto &y : g[x].children) {
				if(g[y].infected) {
					g[y].infected = false;
					segTree.update(1, 1, 2 * n, g[y].in);
				}
			}
		}
		else {
			std::cout << segTree.query(1, 1, 2 * n, g[x].in, g[x].out) << '\n'; 
		}
	}
}
