#include<iostream>
#include<vector>

using namespace std;

const int MAXN = 3e5 + 5;

struct segmentTree
{
    int infected;
    bool lazy;

    int l, r;
    segmentTree *L, *R;

    segmentTree(){}
    segmentTree(int l, int r)
    {
        this->lazy = false;
        this->infected = 0;

        this->l = l;
        this->r = r;
        this->L = nullptr;
        this->R = nullptr;
    }

    void build()
    {
        if(this->l==this->r) return;

        this->L = new segmentTree(this->l, (this->l+this->r)/2);
        this->R = new segmentTree((this->l+this->r)/2+1, this->r);

        this->L->build();
        this->R->build();
    }

    void updateLazy()
    {
        if(this->lazy==true)
        {
            this->infected = 0;
            if(this->l!=this->r)
            {
                this->L->lazy = true;
                this->R->lazy = true;
            }
        }

        this->lazy = false;
    }

    void updateNode(int q)
    {
        //cout << this->l << " " << this->r << " -> " << q << '\n';

        this->updateLazy();
        if(this->l==this->r && this->l==q)
        {
            this->infected++;
            return;
        }
        if(this->r<q || this->l>q) return;

        this->infected++;
        this->L->updateNode(q);
        this->R->updateNode(q);
    }

    void updateSegment(int ql, int qr)
    {
        this->updateLazy();
        if(this->l>=ql && this->r<=qr)
        {
            this->infected = 0;
            if(this->l!=this->r)
            {
                this->L->lazy = true;
                this->R->lazy = true;
            }

            return;
        }
        if(this->r<ql || this->l>qr) return;

        this->L->updateSegment(ql, qr);
        this->R->updateSegment(ql, qr);
        this->infected = this->L->infected + this->R->infected;
    }

    int query(int ql, int qr)
    {
        this->updateLazy();
        if(this->l>=ql && this->r<=qr) return this->infected;
        if(this->r<ql || this->l>qr) return 0;

        return this->L->query(ql, qr) + this->R->query(ql, qr);
    }
};

segmentTree *T;

int n;
int parent[MAXN];
vector <int> graph[MAXN];

int dfsIndex = 0;
int treeSize[MAXN];
int node2Index[MAXN];

void infectNode(int x)
{
    //cout << "INFECT " << x << '\n';

    if(T->query(node2Index[x], node2Index[x])==1) return;
    T->updateNode(node2Index[x]);
}

void clearSegment(int l, int r)
{
    T->updateSegment(l, r);
}

void DFSInit(int x)
{
    treeSize[x] = 1;
    node2Index[x] = dfsIndex;dfsIndex++;

    for(int y: graph[x])
    {
        DFSInit(y);
        treeSize[x] += treeSize[y];
    }

    //cout << x  << " -> " << node2Index[x] << " " << treeSize[x] << '\n';
}

void Init()
{
    T = new segmentTree(0, n-1);
    T->build();

    DFSInit(1);
}

bool help[MAXN];
int findLCA(int u, int v)
{
    int uCopy = u;
    while(u!=-1)
    {
        help[u] = true;
        u = parent[u];
    }

    int ans = -1;
    while(v!=-1)
    {
        if(help[v]==true)
        {
            ans = v;
            break;
        }
        v = parent[v];
    }

    while(uCopy!=-1)
    {
        help[uCopy] = false;
        uCopy = parent[uCopy];
    }
}

bool isPath[MAXN];
vector <int> path;

void makePath(int u)
{
    for(int y: path)
    {
        isPath[y] = false;
    }
    path.clear();

    while(u!=-1)
    {
        path.push_back(u);
        u = parent[u];
    }

    for(int y: path)
    {
        isPath[y] = true;
    }
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n;
    parent[1] = -1;
    for(int i = 2;i<=n;i++)
    {
        cin >> parent[i];
        graph[ parent[i] ].push_back(i);
    }
    Init();

    int Q;
    cin >> Q;

    while(Q--)
    {
        int type;
        cin >> type;

        if(type==1)
        {
            int x;
            cin >> x;

            infectNode(x);
        }
        else if(type==2)
        {
            int u;
            cin >> u;

            makePath(u);
            for(int x: path)
            {
                if(T->query(node2Index[x], node2Index[x])==0) continue;
                //cout << "DO " << x << '\n';

                if(parent[x]!=-1 && isPath[ parent[x] ]==false)
                {
                    infectNode(parent[x]);
                }
                for(int y: graph[x])
                {
                    if(isPath[y]==true) continue;
                    infectNode(y);
                }

                clearSegment(node2Index[x], node2Index[x]);
            }
        }
        else if(type==3)
        {
            int x;
            cin >> x;

            clearSegment(node2Index[x], node2Index[x]);
            for(int y: graph[x])
            {
                clearSegment(node2Index[y], node2Index[y]);
            }
        }
        else if(type==4)
        {
            int x;
            cin >> x;

            cout << T->query(node2Index[x], node2Index[x]+treeSize[x]-1) << '\n';
        }
    }
}
