#include<iostream>
#include<cstring>
#include<vector>

using namespace std;

const int MAXN = 3e5 + 5;

struct segmentTree
{
    int tree[MAXN];

    void build()
    {
        memset(tree, 0, sizeof(tree));
    }

    void update(int index, int val)
    {
        index++;
        while(index<MAXN)
        {
            tree[index] += val;
            index += index&(-index);
        }
    }

    int query(int index)
    {
        index++;
        int sum = 0;

        while(index>0)
        {
            sum += tree[index];
            index -= index&(-index);
        }

        return sum;
    }

    void updateNode(int q)
    {
        update(q, 1);
    }

    void updateSegment(int ql, int qr)
    {
        update(ql, -1);
    }

    int query(int ql, int qr)
    {
        return query(qr) - query(ql-1);
    }
};

segmentTree *T;

int n;
int parent[MAXN];
vector <int> graph[MAXN];

int dfsIndex = 0;
int treeSize[MAXN];
int node2Index[MAXN];

bool isInfected[MAXN];

void infectNode(int x)
{
    //cout << "INFECT " << x << '\n';

    if(isInfected[ node2Index[x] ]==true) return;

    isInfected[ node2Index[x] ] = true;
    T->updateNode(node2Index[x]);
}

void clearSegment(int l, int r)
{
    if(isInfected[l]==false) return;

    isInfected[l] = false;
    T->updateSegment(l, r);
}

void DFSInit(int x)
{
    treeSize[x] = 1;
    node2Index[x] = dfsIndex;dfsIndex++;

    for(int y: graph[x])
    {
        DFSInit(y);
        treeSize[x] += treeSize[y];
    }

    //cout << x  << " -> " << node2Index[x] << " " << treeSize[x] << '\n';
}

void Init()
{
    T = new segmentTree();
    T->build();

    DFSInit(1);
}

bool help[MAXN];
int findLCA(int u, int v)
{
    int uCopy = u;
    while(u!=-1)
    {
        help[u] = true;
        u = parent[u];
    }

    int ans = -1;
    while(v!=-1)
    {
        if(help[v]==true)
        {
            ans = v;
            break;
        }
        v = parent[v];
    }

    while(uCopy!=-1)
    {
        help[uCopy] = false;
        uCopy = parent[uCopy];
    }
}

bool isPath[MAXN];
vector <int> path;

void makePath(int u)
{
    for(int y: path)
    {
        isPath[y] = false;
    }
    path.clear();

    while(u!=-1)
    {
        path.push_back(u);
        u = parent[u];
    }

    for(int y: path)
    {
        isPath[y] = true;
    }
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n;
    parent[1] = -1;
    for(int i = 2;i<=n;i++)
    {
        cin >> parent[i];
        graph[ parent[i] ].push_back(i);
    }
    Init();

    int Q;
    cin >> Q;

    while(Q--)
    {
        int type;
        cin >> type;

        if(type==1)
        {
            int x;
            cin >> x;

            infectNode(x);
        }
        else if(type==2)
        {
            int u;
            cin >> u;

            makePath(u);
            for(int x: path)
            {
                if(T->query(node2Index[x], node2Index[x])==0) continue;
                //cout << "DO " << x << '\n';

                if(parent[x]!=-1 && isPath[ parent[x] ]==false)
                {
                    infectNode(parent[x]);
                }
                for(int y: graph[x])
                {
                    if(isPath[y]==true) continue;
                    infectNode(y);
                }

                clearSegment(node2Index[x], node2Index[x]);
            }
        }
        else if(type==3)
        {
            int x;
            cin >> x;

            clearSegment(node2Index[x], node2Index[x]);
            for(int y: graph[x])
            {
                clearSegment(node2Index[y], node2Index[y]);
            }
        }
        else if(type==4)
        {
            int x;
            cin >> x;

            cout << T->query(node2Index[x], node2Index[x]+treeSize[x]-1) << '\n';
        }
    }
}
