#include <iostream>
#include <vector>
#include <bitset>

#pragma GCC optimize "Ofast"
using std::cin; using std::cout; using std::endl;

const int MAX_N = 5e5 + 10;
int N, M;
struct State {
	std::bitset<MAX_N * 3> impos;

	template<int MC>
	void color(int idx, int color) {
		for(int i = 0;i < MC;i ++) impos[idx*3 + i] = true;
		impos[idx*3 + color] = false;
	}

	template<int MC>
	int cnt(int idx) const {
		return MC
		     - impos[idx*3 + 0]
		     - impos[idx*3 + 1]
		     - impos[idx*3 + 2];
	}
};

std::vector<int> graph[MAX_N];

template<int MC>
bool Color(int idx, int color, State &curr) {
	curr.color<MC>(idx, color);
	if(curr.cnt<MC>(idx) == 0) return false;

	for(auto nei : graph[idx]) {
		if(curr.impos[nei*3 + color]) continue;
		curr.impos[nei*3 + color] = true;

		const int cnt = curr.cnt<MC>(nei);
		if(cnt == 0) return false;

		if(cnt == 1) {
			// find it
			for(int c = 0;c < MC;c ++) {
				if(!curr.impos[nei*3 + c]) {
					bool r = Color<MC>(nei, c, curr);
					if(!r) return false;
					break;
				}
			}
		}
	}

	return true;
}

template<int MC>
void rec(const State &curr) {
	/*
	std::cout << MC << "here" << endl;
	for(int i = 0;i < N;i ++) {
		cout << i << "| ";
		for(int c = 0;c < 3;c ++) cout << curr.impos[i*3 + c] << ' ';
		cout << " = " << curr.cnt<MC>(i);
		cout << '\n';
	} 
	*/

	bool any = false;

	for(int i = 0;i < N;i ++) {
		if(curr.cnt<MC>(i) == 1) continue;
		if(curr.cnt<MC>(i) == 0) {
			std::cerr << "ERROR " << i << endl;
			std::cerr << 1/0 << endl;
		}

		any = true;

		for(int color = 0;color < MC;color ++) {
			if(curr.impos[i*3 + color]) continue;

			// make i coloed color
			State ns = curr;
			if(Color<MC>(i, color, ns)) rec<MC>(ns);
		}	
	}

	if(!any) {
		//std::cerr << "OK" << std::endl;
		cout << MC << '\n';
		for(int i = 0;i < N;i ++) {
			for(int c = 0;c < MC;c ++) {
				if(!curr.impos[i*3 + c]) {
					cout << c+1 << ' ';
					break;
				}
			}
		}
		cout << endl;
		exit(0);
	}
}


int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr);

	cin >> N >> M;
	for(int i = 0;i < N;i ++) {
		graph[i].push_back((i+1) % N);
		graph[(i+1) % N].push_back(i);
	}

	for(int i = 0;i < M;i ++) {
		int u, v;
		cin >> u >> v;
		u --;
		v --;
		graph[u].push_back(v);
		graph[v].push_back(u);
	}

	{
		State initial;
		if(Color<2>(0, 0, initial)) rec<2>(initial);
	}

	{
		State initial;
		Color<3>(0, 0, initial);
		rec<3>(initial);
	}
}