#include <iostream>
#include <vector>

#pragma GCC optimize "Ofast"
using std::cin; using std::cout; using std::endl;

const int MAX_N = 3e5 + 10;
int parent[MAX_N];
std::vector<int> children[MAX_N];
int subtree[MAX_N];
bool inf[MAX_N];
int n;

inline void change_up(int vertex, int diff) {
	while(vertex != -1) {
		subtree[vertex] += diff;
		vertex = parent[vertex];
	}
}

void simple_infest(int vertex) {
	if(inf[vertex]) return;
	inf[vertex] = true;
	change_up(vertex, 1);
}

void path_clean(int vertex) {
	int less = 0;
	int prev = -99;
	while(vertex != -1) {
		if(inf[vertex]) {
			for(auto i : children[vertex]) {
				if(i == prev) continue;
				if(!inf[i]) {
					less --;
					inf[i] = true;
					subtree[i] ++;
				}
			}

			inf[vertex] = false;
			less ++;
		}
		subtree[vertex] -= less;
		prev = vertex;
		vertex = parent[vertex];
	}
}

void nuke(int vertex) {
	if(inf[vertex]) inf[vertex] = false;

	int ns = 0;
	for(auto i : children[vertex]) {
		if(inf[i]) {
			inf[i] = false;
			subtree[i] --;
		}
		ns += subtree[i];
	}

	int diff = ns - subtree[vertex];
	change_up(vertex, diff);
}

int query(int vertex) {
	return subtree[vertex];
}

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr);

	cin >> n;
	parent[0] = -1;
	for(int i = 0;i < n-1;i ++) {
		cin >> parent[i+1];
		parent[i+1] --;
		children[parent[i+1]].push_back(i+1);
	}

	int q;
	cin >> q;
	for(int _ = 0;_ < q;_ ++) {
		int type, vertex;
		cin >> type >> vertex;
		vertex --;
		//cout << "============ after " << _ << endl;
		//cout << "here " << type << ' ' << vertex << endl;
		if(type == 1) {
			simple_infest(vertex);
		}
		if(type == 2) {
			path_clean(vertex);
		}
		if(type == 3) {
			nuke(vertex);
		}
		if(type == 4) {
			cout << query(vertex) << '\n';
		}
		//for(int i = 0;i < n;i ++) {
				//cout << i << ' ' << inf[i] << ' ' << subtree[i] << endl;
		//}
	}
}