#include <bits/stdc++.h>
#define lsb(x) (x & (-x))

using namespace std;

struct Fenwick {
    vector <int> aib;
    int n;
    inline void init(int _n) {
        n = _n;
        aib.resize(n + 1);
    }
    inline void update(int pos, int val) {
        for(int i = pos; i <= n; i += lsb(i)) {
            aib[i] += val;
        }
    }
    inline int query(int pos) {
        int ans = 0;
        for(int i = pos; i > 0; i -= lsb(i)) {
            ans += aib[i];
        }
        return ans;
    }
    inline int sum(int l, int r) {
        return query(r) - query(l - 1);
    }
};

const int MAXN = (int) 3e5;

vector <int> g[MAXN + 1];
int anc[MAXN + 1][20];
int idl[MAXN + 1], idr[MAXN + 1], id_sz;
int first[MAXN + 1], last[MAXN + 1], sz;

void dfs(int nod, int par) {
    anc[nod][0] = par;
    for(int bit = 1; bit < 20; bit++) {
        anc[nod][bit] = anc[anc[nod][bit - 1]][bit - 1];
    }

    idl[nod] = ++id_sz;
    first[nod] = ++sz;
    for(auto it : g[nod]) {
        if(it != par) {
            dfs(it, nod);
        }
    }
    idr[nod] = id_sz;
    last[nod] = ++sz;
}

Fenwick fen_up, fen_down;
set <int> nodes[MAXN + 1];
bool mark[MAXN + 1];

inline void add(int nod) {
    if(mark[nod] == 0) {
        mark[nod] = 1;
        fen_down.update(idl[nod], 1);
        fen_up.update(first[nod], 1);
        fen_up.update(last[nod], -1);
        if(nod != 1) {
            nodes[anc[nod][0]].insert(nod);
        }
    }
}

inline void sub(int nod) {
    if(mark[nod]) {
        mark[nod] = 0;
        fen_down.update(idl[nod], -1);
        fen_up.update(first[nod], -1);
        fen_up.update(last[nod], 1);
        if(nod != 1) {
            nodes[anc[nod][0]].erase(nod);
        }
    }
}

inline int solve(int type, int nod, int n) {
    if(type == 1) {
        add(nod);
    }
    if(type == 2) {
        int cur = nod;
        while(cur != 0) {
            int x = cur;
            for(int bit = 19; bit >= 0; bit--) {
                if(anc[x][bit] > 0 && fen_up.sum(first[anc[x][bit]], first[x]) == 0) {
                    x = anc[x][bit];
                }
            }
            if(mark[cur] == 0) {
                cur = anc[x][0];
            }
            if(cur != 0) {
                //assert(mark[cur]);
                sub(cur);
                for(auto it : g[cur]) {
                    if(it != anc[cur][0]) {
                        if(idl[it] <= idl[nod] && idr[nod] <= idr[it]) continue;
                        add(it);
                    }
                }
            }
        }
    }
    if(type == 3) {
        vector <int> aux;
        for(auto it : nodes[nod]) {
            aux.push_back(it);
        }
        for(auto it : aux) {
            sub(it);
        }
        sub(nod);
        //assert(nodes[nod].size() == 0);
    }
    if(type == 4) {
        return fen_down.sum(idl[nod], idr[nod]);
    }
    return -1;
}

int main() {
#ifdef HOME
    ifstream cin("A.in");
    ofstream cout("A.out");
#endif
    int i, n, q, x;
    ios::sync_with_stdio(false);
    cin.tie(0), cout.tie(0);

    cin >> n;

    for(i = 2; i <= n; i++) {
        int x;
        cin >> x;
        g[x].push_back(i);
    }

    dfs(1, 0);

    fen_up.init(2 * n); fen_down.init(n);

    cin >> q;
    while(q--) {
        int type, nod;
        cin >> type >> nod;
        int ans = solve(type, nod, n);
        if(type == 4) {
            cout << ans << "\n";
        }
        /*for(i = 1; i <= n; i++) {
            cerr << mark[i] << " ";
        }
        cerr << "\n";*/
    }

    return 0;
}
