#include <cstdio>
#include <cassert>
#include <cmath>
#include <climits>
#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>

using namespace std;
#define all(x) x.begin(), x.end()
#define mp make_pair
#define mt make_tuple

const int N = 1e6;
int n;
int par[N];
vector<int> g[N];
int order[N];
int tin[N];
int tout[N];
int timer = 0;

void euler(int v) {
	order[timer] = v;
	tin[v] = timer++;
	for (int u : g[v]) {
		euler(u);
	}
	tout[v] = timer;
}

int tree[2 * N];
int on[N];

void update(int i, int x) {
	tree[i += N] = x;
	for (i /= 2; i > 0; i /= 2) {
		tree[i] = tree[2 * i] + tree[2 * i + 1];
	}
}

int sum(int l, int r) {
	int s = 0;
	for (l += N, r += N; l <= r; l /= 2, r /= 2) {
		if (l % 2 == 1) {
			s += tree[l++];
		}
		if (r % 2 == 0) {
			s += tree[r--];
		}
	}
	return s;
}

int sumquery(int v) {
	return sum(tin[v], tout[v] - 1);
}


void updquery(int v, int x) {
	on[v] = x;
	update(tin[v], x);
}

void ultraquery(int v) {
	int p = -1;
	while (v != -1) {
		if (on[v]) {
			updquery(v, 0);
			for (int u : g[v]) {
				if (u == p) {
					continue;
				}
				updquery(u, 1);
			}
		}
		p = v;
		v = par[v];
	}
}

void sonsquery(int v) {
	updquery(v, 0);
	for (int u : g[v]) {
		updquery(u, 0);
	}
}

int readint() {
	int x;
	cin >> x;
	return x;
}

signed main() {
#ifdef LC
	assert(freopen("input.txt", "r", stdin));
#endif
	ios::sync_with_stdio(0);
	cin.tie(0);

	n = readint();
	par[0] = -1;
	for (int i = 1; i < n; ++i) {
		par[i] = readint() - 1;
		g[par[i]].push_back(i);
	}
	euler(0);

	int q = readint();
	while (q--) {
		int tp = readint();
		if (tp == 1) {
			updquery(readint() - 1, 1);
		} else if (tp == 2) {
			ultraquery(readint() - 1);
		} else if (tp == 3) {
			sonsquery(readint() - 1);
		} else {
			assert(tp == 4);
			cout << sumquery(readint() - 1) << "\n";
		}
		/*
		for (int i = 0; i < n; ++i) {
			cout << on[i];
		}
		cout << endl;
		*/
	}

	return 0;
}
