#include <cstdio>
#include <cassert>
#include <cmath>
#include <climits>
#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>

using namespace std;
#define all(x) x.begin(), x.end()
#define mp make_pair
#define mt make_tuple

const int N = 1e6;
int n;
int par[N];
vector<int> g[N];
set<int> t[N][2];

int order[N];
int tin[N];
int tout[N];
int timer = 0;
int size[N];
int start[N];

void predfs(int v) {
	size[v] = 1;
	for (int u : g[v]) {
		predfs(u);
		size[v] += size[u];
	}
}

void euler(int v, int st) {
	order[timer] = v;
	tin[v] = timer++;
	if (st == -1) {
		start[v] = tin[v];
	} else {
		start[v] = st;
	}
	sort(all(g[v]), [&](int i, int j) {
		return size[i] > size[j];
	});
	for (int u : g[v]) {
		euler(u, u == g[v].front() ? st : -1);
	}
	tout[v] = timer;
}

int tree[2 * N];
int on[N];

void updquery(int i, int x) {
	if (on[i] == x) {
		return;
	}
	int p = par[i];
	if (p != -1) {
		t[p][on[i]].erase(i);
		t[p][x].insert(i);
	}
	on[i] = x;
	i = tin[i] + N;
	tree[i] = x;
	for (i /= 2; i > 0; i /= 2) {
		tree[i] = tree[2 * i] + tree[2 * i + 1];
	}
}

int sum(int l, int r) {
	int s = 0;
	for (l += N, r += N; l <= r; l /= 2, r /= 2) {
		if (l % 2 == 1) {
			s += tree[l++];
		}
		if (r % 2 == 0) {
			s += tree[r--];
		}
	}
	return s;
}

int sumquery(int v) {
	return sum(tin[v], tout[v] - 1);
}

bool isanc(int v, int u) {
	return tin[v] <= tin[u] && tout[u] <= tout[v];
}

int jump(int v) {
//	cerr << "in " << v << endl;
	while (v != -1 && !on[v]) {
		//cerr << v << " " << par[v] << " " << tin[v] << " " << start[v] << endl;
		if (sum(start[v], tin[v]) == 0) {
			//cerr << "fi" << endl;
			v = par[order[start[v]]];
		} else {
			int l = start[v], r = tin[v];
			while (r - l > 1) {
				int mid = (l + r) / 2;
				if (sum(mid, tin[v])) {
					l = mid;
				} else {
					r = mid;
				}
			}
			v = order[l];
			//cerr << "kek " << v << " " << tin[v] << " " << start[v] << endl;
		}
	}
//	cerr << "out " << v << endl;
	return v;
}

void ultraquery(int v) {
	int p = v;
	while (v != -1) {
		if (on[v]) {
			updquery(v, 0);
			vector<int> temp(all(t[v][0]));
			for (int u : temp) {
				if (isanc(u, p)) {
					continue;
				}
				updquery(u, 1);
			}
		}
		v = jump(v);
	}
}

void sonsquery(int v) {
	updquery(v, 0);
	vector<int> temp(all(t[v][1]));
	for (int u : temp) {
		updquery(u, 0);
	}
}

int readint() {
	int x;
	cin >> x;
	return x;
}

signed main() {
#ifdef LC
	assert(freopen("input.txt", "r", stdin));
#endif
	ios::sync_with_stdio(0);
	cin.tie(0);

	n = readint();
	par[0] = -1;
	for (int i = 1; i < n; ++i) {
		par[i] = readint() - 1;
		g[par[i]].push_back(i);
		t[par[i]][0].insert(i);
	}
	predfs(0);
	euler(0, -1);

	int q = readint();
	while (q--) {
		int tp = readint();
		if (tp == 1) {
			updquery(readint() - 1, 1);
		} else if (tp == 2) {
			ultraquery(readint() - 1);
		} else if (tp == 3) {
			sonsquery(readint() - 1);
		} else {
			assert(tp == 4);
			cout << sumquery(readint() - 1) << "\n";
		}
	}

	return 0;
}
